/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.wiki.redirect;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A filter that redirects from /wiki/Wiki.jsp?page=PageToGo to
 * http://labs.jboss.com/wiki/PageToGo (by default, but it's configurable)
 * 
 * @author Tomasz Szymanski
 */

public class WikiRedirectFilter implements Filter {

	/**
	 * Old link begginig
	 */
	private static final String oldLink = "/wiki/Wiki.jsp";

	/**
	 * Host with JBoss Wiki. This is configurable as initParameter
	 */
	private static String redirectURI = "http://labs.jboss.org/wiki/";

	/**
	 * Name of the initParameter to set redirectURI
	 */
	private static final String REDIRECT_URI = "redirectURI";

	/**
	 * Where to redirect when wrong uri is entered. This is configurable as
	 * initParameter
	 */
	private static String errorURI = "/errors/404";

	/**
	 * Name of the initParameter to set errorURI
	 */
	private static final String ERROR_URI = "errorURI";

	public void init(FilterConfig conf) {
		// get REDIRECT_URI parameter
		if (conf.getInitParameter(REDIRECT_URI) != null) {
			redirectURI = conf.getInitParameter(REDIRECT_URI);
		}

		// get ERROR_URI parameter
		if (conf.getInitParameter(ERROR_URI) != null) {
			errorURI = conf.getInitParameter(ERROR_URI);
		}
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		if (request instanceof HttpServletRequest) {
			HttpServletRequest httpRequest = (HttpServletRequest) request;
			HttpServletResponse httpResponse = (HttpServletResponse) response;

			String requestURI = httpRequest.getRequestURI();

			// if requested URI starts with /wiki/Wiki.jsp?page=Page - do the redirection
			if (requestURI.startsWith(oldLink)
					&& httpRequest.getParameter("page") != null) {
				
				// sends redirection to REDIRECT_URI/Page
				httpResponse.sendRedirect(redirectURI
						+ httpRequest.getParameter("page"));
			} else {
				
				// if link is wrong - redirect to error page (ERROR_URI)
				httpResponse.sendRedirect(errorURI);
			}

		} else {
			throw new IOException(
					"This servlet should be called as HTTPServletRequest");
		}
	}

	public void destroy() {

	}
}
