/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.DataSourceException;
import org.jboss.wiki.exceptions.WikiSaveException;

/**
 * <p>
 * </p>
 * 
 */
public class FileMediaDataSource implements MediaDataSource {

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	public final static String SAVE_COMMENT = "Saved by FileDataSource of JBoss Wiki";

	private final static String OTHER_LANG_DIR = "LANG/";

	private String GLOBAL_PROPS = "global.properties";

	private final static String LANG_FILE = "languages.properties";

	public static final String propFileName = "filedatasource.properties";

	private static final String propModFileName = "fileMod.properties";

	private static final String CHARSET = "UTF-8";

	private String pathToMediaTrash;

	private String pathToMedia;

	private Properties pageModProps;

	private File modFile;

	private WikiEngine wikiEngine;

	private Logger log;

	private String langCode = null;

	private Properties globalProps;

	public void setWikiEngine(WikiEngine wikiEngine) {
		this.wikiEngine = wikiEngine;

		if (langCode == null)
			langCode = wikiEngine.defaultLangugeCode;
	}

	public FileMediaDataSource() {
		init("");
	}

	private void init(String prefix) {
		log = Logger.getLogger(FileMediaDataSource.class);

		log.info("Looking for props file !");

		Properties fileDSProps = new Properties();

		try {

			fileDSProps.load(FileMediaDataSource.class
					.getResourceAsStream(propFileName));

		} catch (IOException ioe) {
			log.error("Can't load the file " + propFileName + "\n", ioe);
		}

		loadProperties(fileDSProps, prefix);

		File mainWikiDir = new File(pathToMedia);

		if (!mainWikiDir.exists())
			mainWikiDir.mkdirs();

		pageModProps = new Properties();
		modFile = new File(pathToMedia + "/" + propModFileName);

		FileInputStream fis = null;

		try {
			if (!modFile.exists()) {
				modFile.createNewFile();
			}

			fis = new FileInputStream(modFile);

			pageModProps.load(fis);

		} catch (IOException ioe) {
			log.error("Can't load the file " + propModFileName + "\n", ioe);
		} finally {
			try {
				fis.close();
			} catch (IOException ioe) {
				log.error("Can't close input stream \n", ioe);
			}
		}

		// create trash directory for deleted pages if it doesn't exists

		pathToMediaTrash = pathToMedia + "/" + MediaDataSource.mediaTrashName;

		File pathToTrashFile = new File(pathToMediaTrash + "/OLD");

		if (!pathToTrashFile.exists()) {
			pathToTrashFile.mkdirs();
		}

	}

	private void loadProperties(Properties fileDSProps, String prefix) {
		if (fileDSProps.getProperty("pathToMedia") != null) {
			pathToMedia = fileDSProps.getProperty("pathToMedia")
					+ (prefix.length() > 0 ? "/" + prefix : "");
			log.info("Path to media is: " + pathToMedia);

			File mediaDir = new File(pathToMedia);

			if (!mediaDir.exists()) {
				mediaDir.mkdirs();
				log.info("Dir for attachments created");
			}
		} else {
			System.err
					.println("Your properties file lacks of pathToMedia property. It will not run properly ! Please update "
							+ propFileName);
		}
	}

	public void preSave() throws WikiSaveException {
		// do nothing
	}

	public void savePage(String uid, WikiPage page) throws WikiSaveException {
		String uidTranslated = FileDSCommons.translateToFileName(uid);

		if (pageExists(uid)) { // page exists on

			// harddisk - add new
			// version
			Properties pageProps = getPageProps(uid);

			pageProps.setProperty(page.getLastVersion() + ".author", page
					.getLastAuthor().getName());

			try {
				pageProps.store(new FileOutputStream(new File(pathToMedia
						+ "/OLD/" + uidTranslated + "/page.properties")),
						"Saved by FileDataSource of JBoss Wiki");
			} catch (Exception e) {
				throw new WikiSaveException(e);
			}

			log.info("Opening files");
			File pageFile = new File(pathToMedia + "/OLD/" + uidTranslated
					+ "/" + (page.getLastVersion() - 1) + ".txt");
			File pageOldFile = new File(pathToMedia + "/" + uidTranslated
					+ ".txt");

			PrintWriter pageFileWriter;
			InputStreamReader pageFileReader;

			try {
				pageFileWriter = new PrintWriter(pageFile, CHARSET);
				pageFileReader = new InputStreamReader(new FileInputStream(
						pageOldFile), CHARSET);

				int b;

				while ((b = pageFileReader.read()) != -1) {
					pageFileWriter.write(b);
				}

				pageFile.setLastModified(pageOldFile.lastModified());

				pageFileWriter.close();

				pageOldFile.delete();

				pageFileWriter = new PrintWriter(pageOldFile, CHARSET);

				String pageContent = page.getContent();

				pageFileWriter.write(pageContent);
				/*
				 * for (int i = 0; i < pageContent.length(); i++) {
				 * pageFileWriter.write((int) pageContent.charAt(i)); }
				 */

				pageFileWriter.close();
			} catch (FileNotFoundException fnfe) {
				throw new WikiSaveException(fnfe);
			} catch (IOException ioe) {
				throw new WikiSaveException(ioe);
			}

		} else { // there is no page on disk. We have to create new one.

			try {

				// create needed dirs
				File newPageFile = new File(pathToMedia + "/OLD/"
						+ uidTranslated);
				newPageFile.mkdirs();

				// write the page
				newPageFile = new File(pathToMedia + "/OLD/" + uidTranslated
						+ "/page.properties");

				FileOutputStream pageProps = new FileOutputStream(newPageFile);

				Properties properties = new Properties();
				properties.setProperty(page.getLastVersion() + ".author", page
						.getLastAuthor().getName());

				properties.store(pageProps, SAVE_COMMENT);

				pageProps.close();

				String pageDirName = pathToMedia;
				String[] pageNames = uidTranslated.split("/");

				for (int i = 0; i < pageNames.length - 1; i++) {
					pageDirName += "/" + pageNames[i];
				}

				File pageDir = new File(pageDirName);

				pageDir.mkdirs();

				PrintWriter pageWriter = new PrintWriter(new File(pathToMedia
						+ "/" + uidTranslated + ".txt"), CHARSET);

				String pageContent = page.getContent();

				pageWriter.write(pageContent);

				/*
				 * for (int i = 0; i < pageContent.length(); i++) {
				 * pageWriter.write((int) pageContent.charAt(i)); }
				 */

				pageWriter.close();

			} catch (Exception e) {
				throw new WikiSaveException(e);
			}
		}

	}

	public void postSave() throws WikiSaveException {
		// do nothing
	}

	public boolean preGet() {
		return true;
	}

	public Properties getPageProps(String pageName) {
		Properties pageProps = new Properties();

		String translatedPageName = FileDSCommons.translateToFileName(pageName);
		try {
			FileInputStream fpi = new FileInputStream(new File(pathToMedia
					+ "/OLD/" + translatedPageName + "/page.properties"));
			pageProps.load(fpi);
			fpi.close();
		} catch (IOException ioe) {
			// There is no prop file for some reson - create one.

			File pagePropsFile = new File(pathToMedia + "/OLD/"
					+ translatedPageName);

			// create dirs
			pagePropsFile.mkdirs();

			// create file
			pagePropsFile = new File(pathToMedia + "/OLD/" + translatedPageName
					+ "/page.properties");

			try {
				pagePropsFile.createNewFile();

				if (pageExists(pageName)) {
					// if page exists it looks like there is lack of the props
					// structure
					FileOutputStream fpo = new FileOutputStream(pagePropsFile);

					pageProps.setProperty("1.author", "unknown");
					pageProps.store(fpo, SAVE_COMMENT);
					fpo.close();
				}

			} catch (IOException e) {
				log.error(
						"Cannot create page props file for page: " + pageName,
						e);
			}
			return pageProps;
		}

		return pageProps;
	}

	public void saveRolesSet(String uid, Set<String> roles, int action)
			throws DataSourceException {
		File rolesFile;
		try {
			rolesFile = getRolesFile(uid, action);

			if (rolesFile == null)
				throw new DataSourceException("Wrong action number");

		} catch (IOException e) {
			throw new DataSourceException(e);
		}

		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(rolesFile);

			Properties roleProps = new Properties();

			int i = 1;

			for (String roleName : roles) {
				roleProps.setProperty("Role." + i++, roleName);
			}

			roleProps.store(fos, SAVE_COMMENT);

			fos.close();
		} catch (FileNotFoundException e) {
			throw new DataSourceException(e);
		} catch (IOException e) {
			try {
				fos.close();
			} catch (IOException e1) {
				throw new DataSourceException(e);
			}
			throw new DataSourceException(e);
		}

	}

	private File getRolesFile(String uid, int action) throws IOException {
		String fileName = (action == MediaDataSource.EDITABLE) ? "editRoles"
				: (action == MediaDataSource.VIEWABLE) ? "viewRoles" : null;

		if (fileName == null)
			return null;

		File rolesFile = new File(pathToMedia + "/OLD/"
				+ FileDSCommons.translateToFileName(uid) + "/" + fileName
				+ ".properties");

		if (!rolesFile.exists()) {
			rolesFile.createNewFile();
		}

		return rolesFile;
	}

	private Set<String> getRoles(String uid, int action) throws IOException {
		Set<String> set = new TreeSet<String>();

		File rolesFile = getRolesFile(uid, action);

		if (rolesFile == null)
			return null;

		FileInputStream rolesFileIs = new FileInputStream(rolesFile);

		Properties roleProps = new Properties();

		roleProps.load(rolesFileIs);

		int i = 1;

		while (roleProps.getProperty("Role." + i) != null) {
			set.add(roleProps.getProperty("Role." + i));
			i++;
		}

		rolesFileIs.close();

		return set;
	}

	public WikiPage getPage(String pageName) {
		// log.info("[WIKI:FileDataSource]: Getting new page: " +
		// pageName);

		WikiPage page = null;

		File pageFile = new File(pathToMedia + "/"
				+ FileDSCommons.translateToFileName(pageName) + ".txt");

		if (pageFile.exists()) {

			Properties pageProps = getPageProps(pageName);

			if (pageProps != null) {

				int i = 1;

				// get the author of the latest version of page
				while (pageProps.getProperty(String.valueOf(i + 1) + ".author") != null) {
					i++;
				}

				Credentials author = new SimpleCredentials(pageProps
						.getProperty(String.valueOf(i) + ".author"));

				StringBuilder pageContent = new StringBuilder();

				try {
					InputStreamReader input = new InputStreamReader(
							new FileInputStream(pageFile), CHARSET);
					// FileInputStream input = new FileInputStream(pageFile);

					int nextChr = input.read();

					while (nextChr != -1) {
						pageContent.append((char) nextChr);
						nextChr = input.read();
					}
				} catch (IOException ioe) {
					log.error(ioe);
				}

				Set<String> viewRoles = null;
				Set<String> editRoles = null;

				try {
					editRoles = (getRoles(pageName, EDITABLE));
				} catch (IOException e) {
					log.error("Couldn't read edit roles for page: " + pageName,
							e);
				}
				try {
					viewRoles = (getRoles(pageName, VIEWABLE));
				} catch (IOException e) {
					log.error("Couldn't read view roles for page: " + pageName,
							e);
				}

				int pageMods = getPageMod(pageName);

				page = new WikiPage(pageName, author, pageContent.toString(),
						i, i, new Date(pageFile.lastModified()), wikiEngine,
						(pageMods & VIEWABLE) == VIEWABLE,
						(pageMods & EDITABLE) == EDITABLE, editRoles,
						viewRoles, pageFile.length(), langCode);
			}
		}

		// get Metadata props.
		if (page != null) // page exists at all
			page.setMetaDataProps(getMetadataProps(pageName));

		return page;

	}

	public boolean postGet() {
		return true;
	}

	public WikiPage getPage(String pageName, String languageCode) {
		if (languageCode != null) {
			log
					.info("As for now, languages aren't supported in the wiki. Please do not use language codes");
		}
		return getPage(pageName);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, String uid,
			boolean loadContent, int version) {

		/*
		 * Calendar cal = Calendar.getInstance(); cal.setTime(new Date()); long
		 * oldMilis = cal.getTimeInMillis();
		 */

		WikiPage page = null;

		try {
			page = (WikiPage) originPage.clone();
		} catch (CloneNotSupportedException e) {
			log.error(e);
		}

		if (page != null) {
			if (page.getLastVersion() < version) {
				log.info("There is no version " + version + " of page "
						+ page.getName());
				return null;
			}

			getContentAtVersion(page, uid, loadContent, version);

			page.setLastAuthor(getAuthorAtVersion(uid, version));

			page.setVersion(version);

			page.setEditDate(getDateAtVersion(uid, version));

		}

		/*
		 * cal.setTime(new Date());
		 * 
		 * log.info("PAGE "+page.getName()+" loaded in: "+(cal.getTimeInMillis() -
		 * oldMilis));
		 */
		return page;
	}

	public void getContentAtVersion(WikiPage page, String uid,
			boolean loadContent, int version) {
		StringBuilder ret = null;

		File pageFile = new File(pathToMedia + "/OLD/"
				+ FileDSCommons.translateToFileName(uid) + "/" + version
				+ ".txt");

		if (loadContent) {
			try {

				InputStreamReader input = new InputStreamReader(
						new FileInputStream(pageFile), CHARSET);

				int nextChr = input.read();

				ret = new StringBuilder();

				while (nextChr != -1) {
					ret.append((char) nextChr);
					nextChr = input.read();
				}

				page.setContent(ret.toString());
				page.setLength(ret.length());

			} catch (IOException ioe) {
				log.error(ioe);
			}
		} else {
			page.setLength(pageFile.length());
			page.setContent(null);
		}

	}

	public Date getDateAtVersion(String pageName, int version) {
		Date ret = null;

		File pageFile = new File(pathToMedia + "/OLD/" + pageName + "/"
				+ version + ".txt");

		ret = new Date(pageFile.lastModified());

		return ret;
	}

	public Credentials getAuthorAtVersion(String pageName, int version) {
		return new SimpleCredentials(getPageProps(pageName).getProperty(
				version + ".author"));
	}

	public boolean pageExists(String pageName) {

		File pageFile = new File(pathToMedia + "/"
				+ FileDSCommons.translateToFileName(pageName) + ".txt");

		log.debug("PAGE EXISTS (" + pageName + "): " + pathToMedia + "/"
				+ FileDSCommons.translateToFileName(pageName) + ".txt FILE: "
				+ pageFile.getAbsolutePath());
		if (pageFile.exists()) {
			return true;
		} else {
			return false;
		}
	}

	public Set<String> getAllPageNames() {
		Set<String> attSet = new HashSet<String>();

		getPageNamesFor("", attSet);

		return attSet;
	}

	private void getPageNamesFor(String directory, Set<String> attSet) {
		File attDir = new File(pathToMedia + "/" + directory);

		if (attDir.canRead()) {
			File[] pageFiles = attDir.listFiles();
			int i;

			for (i = 0; i < pageFiles.length; i++) {
				if ((pageFiles[i].isFile())
						&& (pageFiles[i].getName().endsWith(".txt"))) {
					// String[] tokens = pageFiles[i].getName().split("/");
					attSet.add((FileDSCommons.translateToPageName(directory)
							+ "/" + FileDSCommons
							.translateToPageName(pageFiles[i].getName()
									.substring(
											0,
											pageFiles[i].getName().length()
													- ".txt".length())))
							.substring(1));
				}
			}

			for (i = 0; i < pageFiles.length; i++) {
				if ((pageFiles[i].isDirectory())
						&& (!pageFiles[i].getName().endsWith("-att"))) {

					// do not take dirs from OLD, .svn and trash folders
					if ((!pageFiles[i].getAbsolutePath().equals(
							pathToMedia + "/OLD"))
							&& (!pageFiles[i].getAbsolutePath().equals(
									pathToMedia + "/.svn"))
							&& (!pageFiles[i].getAbsolutePath().equals(
									pathToMedia + "/"
											+ MediaDataSource.mediaTrashName))
							&& (!pageFiles[i]
									.getAbsolutePath()
									.equals(
											pathToMedia
													+ "/"
													+ AttachmentDataSource.attTrashName))
							&& (!pageFiles[i]
									.getAbsolutePath()
									.equals(
											pathToMedia
													+ "/"
													+ OTHER_LANG_DIR
															.substring(
																	0,
																	OTHER_LANG_DIR
																			.length() - 1)))) {
						// get names for subpages
						getPageNamesFor(directory + "/"
								+ pageFiles[i].getName(), attSet);
					}
				}
			}
		}
	}

	public Set<String> getPagesFor(String pageName) {
		Set<String> pages = new TreeSet<String>();

		File pageDir = new File(pathToMedia + "/"
				+ FileDSCommons.translateToFileName(pageName));

		if ((pageDir.exists()) && (pageDir.isDirectory())) {
			getPagesInside(pageDir, pages, pageName);
		}

		return pages;
	}

	private void getPagesInside(File pageDir, Set<String> pages, String prefix) {
		File[] subFiles = pageDir.listFiles();

		for (int i = 0; i < subFiles.length; i++) {
			if ((subFiles[i].isFile())
					&& (subFiles[i].getName().endsWith(".txt"))) {
				pages.add(prefix
						+ "/"
						+ FileDSCommons.translateToPageName(subFiles[i]
								.getName().substring(
										0,
										subFiles[i].getName().length()
												- ".txt".length())));
			} else if (subFiles[i].isDirectory()) {
				getPagesInside(subFiles[i], pages, prefix
						+ "/"
						+ FileDSCommons.translateToPageName(subFiles[i]
								.getName()));
			}
		}
	}

	public int getPageMod(String pageName) {
		if (pageModProps.getProperty(FileDSCommons
				.translateToFileName(pageName)) == null) {
			return (EDITABLE + VIEWABLE);
		} else {
			return Integer.valueOf(pageModProps.getProperty(FileDSCommons
					.translateToFileName(pageName)));
		}
	}

	public synchronized void setPageMod(String pageName, int mods) {

		if (mods != (EDITABLE + VIEWABLE)) {
			pageModProps.setProperty(FileDSCommons
					.translateToFileName(pageName), String.valueOf(mods));
		} else {
			pageModProps.remove(FileDSCommons.translateToFileName(pageName));
		}

		FileOutputStream fos = null;

		try {
			fos = new FileOutputStream(modFile);
			pageModProps.store(fos, SAVE_COMMENT);
		} catch (IOException ioe) {
			log.error("Couldn't store mod props: ", ioe);
		} finally {
			try {
				fos.close();
			} catch (IOException ioe) {
				log.error("Can't close input stream \n", ioe);
			}
		}
	}

	public synchronized boolean deletePage(String pageName) {

		if (pageExists(pageName)) {
			// moving page file
			String pageFileName = pathToMedia + "/" + pageName + ".txt";
			String pageFileTrashNameBase = pathToMediaTrash + "/" + pageName
					+ ".txt";
			String pageFileTrashName = pageFileTrashNameBase;

			File pageFile = new File(pageFileName);
			File pageTrashFile = new File(pageFileTrashName);

			// create dirs to trash
			File trashFileDir = new File(pathToMediaTrash + "/" + pageName);
			if (!trashFileDir.exists()) {
				trashFileDir.mkdirs();
			}

			// look if page is already in trash (add i-bak)
			int i = 0;

			if (pageTrashFile.exists()) {
				while (pageTrashFile.exists()) {
					pageFileTrashName = pageFileTrashNameBase + "-" + (i++)
							+ "-bak";
					pageTrashFile = new File(pageFileTrashName);
				}

			}

			if (!pageFile.renameTo(pageTrashFile)) {
				log.error("[FileDataSource]: Problems with moving "
						+ pageFileName + " to " + pageFileTrashName);
				return false;
			}

			// moving history directory

			String historyDirName = pathToMedia + "/OLD/" + pageName;
			String historyTrashDirNameBase = pathToMediaTrash + "/OLD/"
					+ pageName;
			String historyTrashDirName = historyTrashDirNameBase;

			// create dirs to trash (history)
			trashFileDir = new File(pathToMediaTrash + "/OLD/" + pageName);
			if (!trashFileDir.exists()) {
				trashFileDir.mkdirs();
			}

			pageFile = new File(historyDirName);
			pageTrashFile = new File(historyTrashDirName);

			// look if history dir is already in trash (add i-bak)
			i = 0;

			if (pageTrashFile.exists()) {
				while (pageTrashFile.exists()) {
					historyTrashDirName = historyTrashDirNameBase + "-" + (i++)
							+ "-bak";
					pageTrashFile = new File(historyTrashDirName);
				}

			}

			if (!pageFile.renameTo(pageTrashFile)) {
				log.error("[FileDataSource]: Problems with moving "
						+ historyDirName + " to " + historyTrashDirName);

				// return back the page history
				pageFile = new File(pageFileName);
				pageTrashFile = new File(pageFileTrashName);

				if (!pageTrashFile.renameTo(pageFile)) {
					System.err
							.println("[FileDataSource]: ERROR. I've tried moving previously moved page file from the trash, but there was some problem. This is not good and there is nothing i can do about it. The error ocured while moving "
									+ pageFileTrashName + " to " + pageFileName);
				}

				return false;
			}

			return true;
		} else {
			// no such page
			log.error("There is now such page: " + pageName);
			return false;
		}
	}

	public Properties getMetadataProps(String uid) {

		if (!pageExists(uid)) {
			// don't get matedata props for nonexisting page
			return new Properties();
		}

		String uidTranslated = FileDSCommons.translateToFileName(uid);

		File propsFile = new File(pathToMedia + "/OLD/" + uidTranslated
				+ "/metadata.properties");

		if (!propsFile.exists()) {

			// create file if it doesn't exist yet
			try {
				propsFile.createNewFile();
			} catch (IOException e) {
				log.error(e);
			}
		}

		Properties props = new Properties();
		FileInputStream fis = null;

		try {
			fis = new FileInputStream(propsFile);
		} catch (FileNotFoundException e1) {
			log.error(e1);
		}

		try {
			props.load(fis);
		} catch (IOException e) {
			log.error(e);
		}

		try {
			fis.close();
		} catch (IOException e) {
			log.error(e);
		}

		return props;
	}

	public void saveMetadataProps(String uid, Properties props) {
		getMetadataProps(uid); // this will create file if needed.

		File propsFile = new File(pathToMedia + "/OLD/"
				+ FileDSCommons.translateToFileName(uid)
				+ "/metadata.properties");

		FileOutputStream fos = null;

		try {
			fos = new FileOutputStream(propsFile);
		} catch (FileNotFoundException e) {
			log.error(e);
		}

		try {
			props.store(fos, SAVE_COMMENT);
		} catch (IOException e) {
			log.error(e);
		}

		try {
			fos.close();
		} catch (IOException e) {
			log.error(e);
		}
	}

	public void setLanguageCode(String langCode) {
		init(OTHER_LANG_DIR + langCode);

		this.langCode = langCode;
	}

	public Map<String, String> getAvailableLanguages() {
		Properties props = new Properties();

		Map<String, String> map = new TreeMap<String, String>();

		map.put(wikiEngine.defaultLangugeCode, wikiEngine.getAllLanguageCodes()
				.get(wikiEngine.defaultLangugeCode));

		try {
			File f = new File(pathToMedia + "/" + LANG_FILE);

			if (!f.exists()) {
				f.createNewFile();
			}

			FileInputStream fis = new FileInputStream(f);

			props.load(fis);

			HashMap mapd = new HashMap(props);

			map.putAll((Map<String, String>) mapd);
		} catch (IOException e) {
			log.error("Couldn't load properties with language list", e);
		}

		return map;
	}

	public void addLangCode(String langCode, String langDesc) {
		Properties props = new Properties();

		props.putAll(getAvailableLanguages());

		props.put(langCode, langDesc);

		try {
			File f = new File(pathToMedia + "/" + LANG_FILE);

			FileOutputStream fos = new FileOutputStream(f);

			props.store(fos, SAVE_COMMENT);
		} catch (IOException e) {
			log.error("Couldn't save properties with language list", e);
		}

	}

	public String getLangCode() {
		return langCode;
	}

	private void refreshGlobalProps() throws IOException {
		if (globalProps == null) {
			globalProps = new Properties();
		}
		File f = new File(pathToMedia + "/" + GLOBAL_PROPS);

		if (!f.exists()) {
			f.createNewFile();
		}

		FileInputStream fis = new FileInputStream(f);

		globalProps.load(fis);

		fis.close();
	}

	public synchronized void setGlobalProperty(String key, String value)
			throws DataSourceException {
		try {
			refreshGlobalProps();

			globalProps.setProperty(key, value);

			saveGlobalProps();
		} catch (IOException e) {
			throw new DataSourceException(e);
		}
	}

	private void saveGlobalProps() throws IOException {
		if (globalProps == null) {
			globalProps = new Properties();
		}
		File f = new File(pathToMedia + "/" + GLOBAL_PROPS);

		if (!f.exists()) {
			f.createNewFile();
		}

		FileOutputStream fos = new FileOutputStream(f);

		globalProps.store(fos, SAVE_COMMENT);

		fos.close();
	}

	public synchronized String getGlobalProperty(String key)
			throws DataSourceException {
		try {
			refreshGlobalProps();

			return globalProps.getProperty(key);
		} catch (IOException e) {
			throw new DataSourceException(e);
		}
	}

}
