package org.jboss.wiki;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.WikiException;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class WikiTypeLoader {

	private WikiEngine wikiEngine;

	private Logger log = Logger.getLogger(this.getClass());

	public WikiTypeLoader(WikiEngine wikiEngine) {
		this.wikiEngine = wikiEngine;
	}

	public void parse(InputStream xmlFile) throws ParserConfigurationException,
			SAXException, IOException, DOMException, InstantiationException,
			IllegalAccessException, ClassNotFoundException {
		DocumentBuilder db = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();

		Document doc = db.parse(xmlFile);

		NodeList wikiTypes = doc.getElementsByTagName("wikiType");

		for (int i = 0; i < wikiTypes.getLength(); i++) {
			WikiType wikiType = null;

			String wikiTypeName = null;

			Node wikiTypeNode = wikiTypes.item(i);

			NodeList wikiTypeElements = wikiTypeNode.getChildNodes();

			for (int j = 0; j < wikiTypeElements.getLength(); j++) {

				Node elem = wikiTypeElements.item(j);

				if (elem.getNodeName().equals("name")) {
					if (wikiType != null)
						wikiType.setName(unmarshallText(elem));
					else
						wikiTypeName = unmarshallText(elem);
				} else if (elem.getNodeName().equals("class")) {
					wikiType = (WikiType) Thread.currentThread()
							.getContextClassLoader().loadClass(
									unmarshallText(elem)).newInstance();
					wikiType.setWikiEngine(wikiEngine);

					if (wikiTypeName != null)
						wikiType.setName(wikiTypeName);
				} else if (elem.getNodeName().equals("plugin")) {
					NodeList pluginElems = elem.getChildNodes();

					String pluginName = null;
					String pluginClass = null;

					for (int k = 0; k < pluginElems.getLength(); k++) {
						Node pluginElem = pluginElems.item(k);

						if (pluginElem.getNodeName().equals("name")) {
							pluginName = unmarshallText(pluginElem);
						} else if (pluginElem.getNodeName().equals("class")) {
							pluginClass = unmarshallText(pluginElem);
						}
					}

					if (pluginName != null && pluginClass != null)
						wikiType.addPlugin(pluginName, pluginClass);
					else
						log.error("Cannot read plugin: name: " + pluginName
								+ " class: " + pluginClass + " in WikiType: "
								+ wikiType.getName());
				}
			}

			wikiEngine.addWikiType(wikiType.getName(), wikiType);
			log.info("Added WikiType: " + wikiType.getName());
		}
	}

	private String unmarshallText(Node textNode) {
		StringBuffer buf = new StringBuffer();

		Node n;
		NodeList nodes = textNode.getChildNodes();

		for (int i = 0; i < nodes.getLength(); i++) {
			n = nodes.item(i);

			if (n.getNodeType() == Node.TEXT_NODE) {
				buf.append(n.getNodeValue());
			}
		}

		return buf.toString();
	}
}
