/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiAttachment;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class AttachementInfoPlugin extends WikiPlugin {

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		Credentials credentials = (Credentials) wikiSession
				.getAttribute("credentials");
		StringBuilder history = new StringBuilder();
		String attachementName = (String) wikiSession.getAttribute("var1");
		Object rollbackToVer = wikiSession.getAttribute("var2");

		String attURL = (String) wikiSession.getAttribute("hostURL") + "/wiki/"
				+ wikiPage.getName() + "/file:";
		String actionURL = (String) wikiSession.getAttribute("actionURL");

		if (((wikiPage.isEditable(credentials)) || (credentials.isAdmin()))
				&& (rollbackToVer != null)
				&& (credentials.isLoggedIn())
				&& (Integer.valueOf((String) rollbackToVer) < wikiEngine
						.getLastAttachmentVersion(wikiPage.getName(),
								attachementName, wikiPage.getLangCode()))) {
			WikiAttachment wikiAttachment = wikiEngine.getAttachment(wikiPage
					.getName(), attachementName, Integer
					.valueOf((String) rollbackToVer), wikiPage.getLangCode());
			wikiEngine.addAttachment(wikiAttachment.getInputStream(),
					attachementName, wikiPage, ((Credentials) wikiSession
							.getAttribute("credentials")).getName(), wikiPage
							.getLangCode());
		}

		history.append("<h3>History of file: ").append(attachementName).append(
				" page: ").append(wikiPage.getName()).append("</h3>\n");
		history.append("<hr>\n").append("<table border=\"1\">\n");
		history
				.append("<tr><td><b>Version</b></td><td><b>Date</b></td><td><b>Author</b></td><td><b>Size</b></td><td><b>Rollback</b></td></tr>\n");

		int lastVer = wikiEngine.getLastAttachmentVersion(wikiPage.getName(),
				attachementName, wikiPage.getLangCode());
		for (int i = lastVer; i > 0; i--) {
			WikiAttachment wikiAttachment = wikiEngine.getAttachment(wikiPage
					.getName(), attachementName, i, wikiPage.getLangCode());
			history.append("<tr><td>").append("<a href=\"").append(attURL)
					.append(attachementName).append(
							(i != lastVer) ? ":" + i : "").append("\" >")
					.append(i).append("</a></td>").append("<td>").append(
							wikiAttachment.getDate()).append("</td>").append(
							"<td>").append(wikiAttachment.getUser()).append(
							"</td>").append("<td>").append(
							wikiAttachment.getSize()).append("</td>");

			if (i != lastVer) {
				history.append("<td>");

				if ((credentials.isLoggedIn())
						&& ((wikiPage.isEditable(credentials)) || (credentials
								.isAdmin())))
					history.append("<a href=\"").append(actionURL).append(
							"&type=fileInfo&page=").append(wikiPage.getName())
							.append("&var1=").append(attachementName).append(
									"&var2=").append(i).append("\" >");

				if (wikiPage.isEditable(credentials)) {
					history
							.append((!credentials.isLoggedIn()) ? "<i>" : "")
							.append("Rollback version ")
							.append(i)
							.append(
									(!credentials.isLoggedIn()) ? " (after login)</i>"
											: "");
				} else {
					history.append("<i>This page IS NOT editable</i>");
				}

				if ((credentials.isLoggedIn())
						&& ((wikiPage.isEditable(credentials)) || (credentials
								.isAdmin())))
					history.append("</a>");

				history.append("<td>");
			} else {
				history.append("<td></td>");
			}

			history.append("</tr>\n");
		}
		history.append("</table>\n");
		wikiPage.setContent(history.toString());

		return wikiPage;
	}

	@Override
	public void init() {
		// TODO Auto-generated method stub

	}

}
