package org.jboss.wiki.plugins;

import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class GetRefsFromPagePlugin extends WikiPlugin {

	private static final String pattern1 = "\\[.+\\]";

	private static final String pattern2 = "\\[[^]|]+\\|.+\\]";

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {

		Set<String> refs = (Set<String>) wikiPage.getPermVariable("refs");

		if (refs == null || updateRefs()) {
			refs = getRefsFrom(wikiPage.getContent(), wikiPage.getLangCode());
			wikiPage.addPermVariable("refs", refs);
		}

		return wikiPage;
	}

	@Override
	public void init() {

	}

	protected boolean updateRefs() {
		return false;
	}

	private Set<String> getRefsFrom(String pageContent, String langCode) {

		Set<String> refs = new TreeSet<String>();

		Matcher matcher = Pattern.compile(pattern1).matcher(pageContent);

		while (matcher.find()) {
			String pageUid = pageContent.substring(matcher.start(), matcher
					.end());

			pageUid = pageUid.substring(1, pageUid.indexOf(']'));

			pageUid = checkRef(pageUid);

			if (pageUid != null)
				refs.add(wikiEngine.getRealName(pageUid, langCode));
		}

		matcher = Pattern.compile(pattern2).matcher(pageContent);

		while (matcher.find()) {
			String pageUid = pageContent.substring(matcher.start(), matcher
					.end());

			pageUid = pageUid.substring(pageUid.indexOf('|') + 1, pageUid
					.indexOf(']'));

			pageUid = checkRef(pageUid);

			if (pageUid != null)
				refs.add(wikiEngine.getRealName(pageUid, langCode));
		}

		return refs;
	}

	private String checkRef(String pageUid) {

		pageUid = pageUid.trim();

		if (!HTMLTranslatorParts.checkImageLink(pageUid)
				&& !HTMLTranslatorParts.isExternalLink(pageUid)
				&& !HTMLTranslatorParts.isNumber(pageUid)) {
			pageUid = HTMLTranslatorParts.cleanLink(pageUid);

			if (pageUid != null && !HTMLTranslatorParts.isNumber(pageUid)) {
				return pageUid;
			}
		}

		return null;
	}

}
