/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class OtherLangsPlugin extends WikiPlugin {

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {

		// there must be more then one language used
		if (wikiEngine.getUsedLanguageCodes().keySet().size() > 1) {

			Credentials credentials = (Credentials) wikiSession
					.getAttribute(WikiSession.CREDENTIALS);

			StringBuilder content = new StringBuilder(wikiPage.getContent())
					.append(HTMLTranslatorParts.pagePartRegexStart).append(
							"\n----\n").append(
							HTMLTranslatorParts.pagePartRegexEnd);

			content.append("<h3>Other languages:");

			if (credentials.isLoggedIn() && wikiPage.isEditable(credentials)) {
				content.append(" <a href=\"").append(
						wikiSession.getAttribute(WikiSession.ACTION_URL))
						.append("&page=").append(wikiPage.getName()).append(
								"&type=linkPage\">[MAKE LINK]</a></h3>\n");
			} else {
				content
						.append("</h3><i>Log in to make links between pages</i><br />\n");
			}

			content.append(HTMLTranslatorParts.pagePartRegexStart);

			String otherLangs = wikiPage
					.getMetaDataProperty(WikiPage.OTHER_LANGS);

			if (otherLangs != null) {
				String hostURL = (String) wikiSession
						.getAttribute(WikiSession.HOST_URL);

				String[] otherLangsArray = otherLangs.split("[,]");
				for (int i = 0; i < otherLangsArray.length; i += 2) {
					content.append("[").append(hostURL).append("/wiki/")
							.append(otherLangsArray[i].toLowerCase()).append(
									"/").append(
									wikiEngine.getRealName(
											otherLangsArray[i + 1],
											otherLangsArray[i])).append("]")
							.append(" ").append("''").append(
									wikiEngine.getAllLanguageCodes().get(
											otherLangsArray[i])).append("'' ");
				}
			}

			content.append(HTMLTranslatorParts.pagePartRegexEnd);

			wikiPage.setContent(content.toString());

		}
		return wikiPage;
	}

	@Override
	public void init() {

	}

}
