/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.management;

import javax.ejb.Local;
import javax.ejb.Remote;

import org.jboss.annotation.ejb.Management;
import org.jboss.annotation.ejb.Service;
import org.jboss.logging.Logger;
import org.jboss.wiki.WikiCommon;
import org.jboss.wiki.WikiEngine;
import org.jboss.wiki.management.WikiServiceManagement;

@Service(objectName = WikiCommon.WIKI_SERVICE_NAME)
@Local(WikiServiceLocal.class)
@Remote(WikiServiceRemote.class)
@Management(WikiServiceManagement.class)
public class WikiService implements WikiServiceLocal, WikiServiceRemote,
		WikiServiceManagement {

	private WikiEngine wikiEngine;

	private Logger log = Logger.getLogger(WikiService.class);

	public synchronized WikiEngine getWikiEngine() {
		if (wikiEngine == null) {
			wikiEngine = new WikiEngine();
		}
		return wikiEngine;
	}

	public void create() throws Exception {
		log.info("WikiService - Creating");
	}

	public void start() throws Exception {
		log.info("WikiService - Starting");
	}

	public void stop() {
		log.info("WikiService - Stoping");
	}

	public void destroy() {
		log.info("WikiService - Destroying");
	}

}
